@@ -21,39 +21,42 @@
 
 #include <ags/X/ags_ui_provider.h>
 #include <ags/X/ags_wave_editor.h>
+#include <ags/X/ags_composite_editor.h>
+
+#include <ags/X/editor/ags_composite_edit.h>
 
 #include <math.h>
 #include <gdk/gdkkeysyms.h>
 
-void ags_wave_edit_drawing_area_button_press_position_cursor(AgsWaveEditor *wave_editor,
-							     AgsWaveToolbar *wave_toolbar,
+void ags_wave_edit_drawing_area_button_press_position_cursor(GtkWidget *editor,
+							     GtkWidget *toolbar,
 							     AgsWaveEdit *wave_edit,
 							     AgsMachine *machine,
 							     GdkEventButton *event);
-void ags_wave_edit_drawing_area_button_press_select_buffer(AgsWaveEditor *wave_editor,
-							   AgsWaveToolbar *wave_toolbar,
+void ags_wave_edit_drawing_area_button_press_select_buffer(GtkWidget *editor,
+							   GtkWidget *toolbar,
 							   AgsWaveEdit *wave_edit,
 							   AgsMachine *machine,
 							   GdkEventButton *event);
 
-void ags_wave_edit_drawing_area_button_release_position_cursor(AgsWaveEditor *wave_editor,
-							       AgsWaveToolbar *wave_toolbar,
+void ags_wave_edit_drawing_area_button_release_position_cursor(GtkWidget *editor,
+							       GtkWidget *toolbar,
 							       AgsWaveEdit *wave_edit,
 							       AgsMachine *machine,
 							       GdkEventButton *event);
-void ags_wave_edit_drawing_area_button_release_select_buffer(AgsWaveEditor *wave_editor,
-							     AgsWaveToolbar *wave_toolbar,
+void ags_wave_edit_drawing_area_button_release_select_buffer(GtkWidget *editor,
+							     GtkWidget *toolbar,
 							     AgsWaveEdit *wave_edit,
 							     AgsMachine *machine,
 							     GdkEventButton *event);
 
-void ags_wave_edit_drawing_area_motion_notify_position_cursor(AgsWaveEditor *wave_editor,
-							      AgsWaveToolbar *wave_toolbar,
+void ags_wave_edit_drawing_area_motion_notify_position_cursor(GtkWidget *editor,
+							      GtkWidget *toolbar,
 							      AgsWaveEdit *wave_edit,
 							      AgsMachine *machine,
 							      GdkEventMotion *event);
-void ags_wave_edit_drawing_area_motion_notify_select_buffer(AgsWaveEditor *wave_editor,
-							    AgsWaveToolbar *wave_toolbar,
+void ags_wave_edit_drawing_area_motion_notify_select_buffer(GtkWidget *editor,
+							    GtkWidget *toolbar,
 							    AgsWaveEdit *wave_edit,
 							    AgsMachine *machine,
 							    GdkEventMotion *event);
@@ -74,12 +77,12 @@ ags_wave_edit_drawing_area_configure_event(GtkWidget *widget, GdkEventConfigure
   
   gtk_widget_queue_draw(wave_edit);
   
-  return(TRUE);
+  return(FALSE);
 }
 
 void
-ags_wave_edit_drawing_area_button_press_position_cursor(AgsWaveEditor *wave_editor,
-							AgsWaveToolbar *wave_toolbar,
+ags_wave_edit_drawing_area_button_press_position_cursor(GtkWidget *editor,
+							GtkWidget *toolbar,
 							AgsWaveEdit *wave_edit,
 							AgsMachine *machine,
 							GdkEventButton *event)
@@ -88,10 +91,24 @@ ags_wave_edit_drawing_area_button_press_position_cursor(AgsWaveEditor *wave_edit
     
   GtkAllocation allocation;
 
+  AgsApplicationContext *application_context;
+  
+  gboolean use_composite_editor;
   gdouble c_range;
   guint g_range;
   double zoom_factor;
 
+  application_context = ags_application_context_get_instance();
+
+  use_composite_editor = ags_ui_provider_use_composite_editor(AGS_UI_PROVIDER(application_context));
+
+  /* zoom */
+  if(use_composite_editor){
+    zoom_factor = exp2(6.0 - (double) gtk_combo_box_get_active((GtkComboBox *) AGS_COMPOSITE_TOOLBAR(toolbar)->zoom));
+  }else{
+    zoom_factor = exp2(6.0 - (double) gtk_combo_box_get_active((GtkComboBox *) AGS_WAVE_TOOLBAR(toolbar)->zoom));
+  }
+  
   gtk_widget_get_allocation(GTK_WIDGET(wave_edit->drawing_area),
 			    &allocation);
 
@@ -101,9 +118,6 @@ ags_wave_edit_drawing_area_button_press_position_cursor(AgsWaveEditor *wave_edit
 
   g_range = gtk_adjustment_get_upper(vscrollbar_adjustment) + allocation.height;
 
-  /* zoom */
-  zoom_factor = exp2(6.0 - (double) gtk_combo_box_get_active((GtkComboBox *) wave_toolbar->zoom));
-
   /* cursor position */
   wave_edit->cursor_position_x = (guint) ((zoom_factor * event->x + gtk_range_get_value(GTK_RANGE(wave_edit->hscrollbar))));
     
@@ -114,17 +128,28 @@ ags_wave_edit_drawing_area_button_press_position_cursor(AgsWaveEditor *wave_edit
 }
 
 void
-ags_wave_edit_drawing_area_button_press_select_buffer(AgsWaveEditor *wave_editor,
-						      AgsWaveToolbar *wave_toolbar,
+ags_wave_edit_drawing_area_button_press_select_buffer(GtkWidget *editor,
+						      GtkWidget *toolbar,
 						      AgsWaveEdit *wave_edit,
 						      AgsMachine *machine,
 						      GdkEventButton *event)
 {
+  AgsApplicationContext *application_context;
+  
+  gboolean use_composite_editor;
   double zoom_factor;
 
-  /* zoom */
-  zoom_factor = exp2(6.0 - (double) gtk_combo_box_get_active((GtkComboBox *) wave_toolbar->zoom));
+  application_context = ags_application_context_get_instance();
 
+  use_composite_editor = ags_ui_provider_use_composite_editor(AGS_UI_PROVIDER(application_context));
+
+  /* zoom */
+  if(use_composite_editor){
+    zoom_factor = exp2(6.0 - (double) gtk_combo_box_get_active((GtkComboBox *) AGS_COMPOSITE_TOOLBAR(toolbar)->zoom));
+  }else{
+    zoom_factor = exp2(6.0 - (double) gtk_combo_box_get_active((GtkComboBox *) AGS_WAVE_TOOLBAR(toolbar)->zoom));
+  }
+  
   wave_edit->selection_x0 = (guint) zoom_factor * event->x + gtk_range_get_value(GTK_RANGE(wave_edit->hscrollbar));
   wave_edit->selection_x1 = wave_edit->selection_x0;
     
@@ -137,47 +162,87 @@ ags_wave_edit_drawing_area_button_press_select_buffer(AgsWaveEditor *wave_editor
 gboolean
 ags_wave_edit_drawing_area_button_press_event(GtkWidget *widget, GdkEventButton *event, AgsWaveEdit *wave_edit)
 {
-  AgsWaveEditor *wave_editor;
-  AgsWaveToolbar *wave_toolbar;
+  GtkWidget *editor;
+  GtkWidget *toolbar;
   AgsMachine *machine;
 
-  wave_editor = (AgsWaveEditor *) gtk_widget_get_ancestor(GTK_WIDGET(wave_edit),
-							  AGS_TYPE_WAVE_EDITOR);
+  AgsApplicationContext *application_context;
+  
+  gboolean use_composite_editor;
+  gboolean selected_position_cursor, selected_select;
 
-  wave_toolbar = wave_editor->wave_toolbar;
+  application_context = ags_application_context_get_instance();
 
+  use_composite_editor = ags_ui_provider_use_composite_editor(AGS_UI_PROVIDER(application_context));
+
+  selected_position_cursor = FALSE;
+  selected_select = FALSE;
+  
+  if(use_composite_editor){
+    AgsCompositeToolbar *composite_toolbar;
+    
+    editor = gtk_widget_get_ancestor(GTK_WIDGET(wave_edit),
+				     AGS_TYPE_COMPOSITE_EDITOR);
+    
+    toolbar = AGS_COMPOSITE_EDITOR(editor)->toolbar;
+
+    machine = AGS_COMPOSITE_EDITOR(editor)->selected_machine;
+
+    composite_toolbar = (AgsCompositeToolbar *) toolbar;
+    
+    selected_position_cursor = (composite_toolbar->selected_tool == composite_toolbar->position) ? TRUE: FALSE;
+    selected_select = (composite_toolbar->selected_tool == composite_toolbar->select) ? TRUE: FALSE;
+
+    AGS_COMPOSITE_EDITOR(editor)->wave_edit->focused_edit = wave_edit;
+  }else{
+    AgsWaveToolbar *wave_toolbar;
+    
+    editor = gtk_widget_get_ancestor(GTK_WIDGET(wave_edit),
+				     AGS_TYPE_WAVE_EDITOR);
+    
+    toolbar = AGS_WAVE_EDITOR(editor)->wave_toolbar;
+
+    machine = AGS_WAVE_EDITOR(editor)->selected_machine;
+
+    wave_toolbar = (AgsWaveToolbar *) toolbar;
+    
+    selected_position_cursor = (wave_toolbar->selected_edit_mode == wave_toolbar->position) ? TRUE: FALSE;
+    selected_select = (wave_toolbar->selected_edit_mode == wave_toolbar->select) ? TRUE: FALSE;
+    
+    AGS_WAVE_EDITOR(editor)->focused_wave_edit = wave_edit;
+  }
+  
   gtk_widget_grab_focus((GtkWidget *) wave_edit->drawing_area);
-  wave_editor->focused_wave_edit = wave_edit;
 
-  if((machine = wave_editor->selected_machine) != NULL &&
+  if(machine != NULL &&
      event->button == 1){    
     wave_edit->button_mask = AGS_WAVE_EDIT_BUTTON_1;
     
-    if(wave_toolbar->selected_edit_mode == wave_toolbar->position){
+    if(selected_position_cursor){
       wave_edit->mode = AGS_WAVE_EDIT_POSITION_CURSOR;
 
-      ags_wave_edit_drawing_area_button_press_position_cursor(wave_editor,
-							      wave_toolbar,
+      ags_wave_edit_drawing_area_button_press_position_cursor(editor,
+							      toolbar,
 							      wave_edit,
 							      machine,
 							      event);
-    }else if(wave_toolbar->selected_edit_mode == wave_toolbar->select){
+    }else if(selected_select){
       wave_edit->mode = AGS_WAVE_EDIT_SELECT_BUFFER;
 
-      ags_wave_edit_drawing_area_button_press_select_buffer(wave_editor,
-							    wave_toolbar,
+      ags_wave_edit_drawing_area_button_press_select_buffer(editor,
+							    toolbar,
 							    wave_edit,
 							    machine,
 							    event);
     }
   }
   
-  return(TRUE);
+  return(FALSE);
 }
 
 void
-ags_wave_edit_drawing_area_button_release_position_cursor(AgsWaveEditor *wave_editor,
-							  AgsWaveToolbar *wave_toolbar,
+ags_wave_edit_drawing_area_button_release_position_cursor(GtkWidget *editor,
+							  GtkWidget *toolbar,
 							  AgsWaveEdit *wave_edit,
 							  AgsMachine *machine,
 							  GdkEventButton *event)
@@ -186,10 +251,24 @@ ags_wave_edit_drawing_area_button_release_position_cursor(AgsWaveEditor *wave_ed
 
   GtkAllocation allocation;
 
+  AgsApplicationContext *application_context;
+  
+  gboolean use_composite_editor;
   gdouble c_range;
   guint g_range;
   double zoom_factor;
 
+  application_context = ags_application_context_get_instance();
+
+  use_composite_editor = ags_ui_provider_use_composite_editor(AGS_UI_PROVIDER(application_context));
+  
+  /* zoom */
+  if(use_composite_editor){
+    zoom_factor = exp2(6.0 - (double) gtk_combo_box_get_active((GtkComboBox *) AGS_COMPOSITE_TOOLBAR(toolbar)->zoom));
+  }else{
+    zoom_factor = exp2(6.0 - (double) gtk_combo_box_get_active((GtkComboBox *) AGS_WAVE_TOOLBAR(toolbar)->zoom));
+  }
+  
   gtk_widget_get_allocation(GTK_WIDGET(wave_edit->drawing_area),
 			    &allocation);
 
@@ -199,9 +278,6 @@ ags_wave_edit_drawing_area_button_release_position_cursor(AgsWaveEditor *wave_ed
 
   g_range = gtk_adjustment_get_upper(vscrollbar_adjustment) + allocation.height;
 
-  /* zoom */
-  zoom_factor = exp2(6.0 - (double) gtk_combo_box_get_active((GtkComboBox *) wave_toolbar->zoom));
-
   /* cursor position */
   wave_edit->cursor_position_x = (guint) ((zoom_factor * event->x + gtk_range_get_value(GTK_RANGE(wave_edit->hscrollbar))));
     
@@ -212,8 +288,8 @@ ags_wave_edit_drawing_area_button_release_position_cursor(AgsWaveEditor *wave_ed
 }
 
 void
-ags_wave_edit_drawing_area_button_release_select_buffer(AgsWaveEditor *wave_editor,
-							AgsWaveToolbar *wave_toolbar,
+ags_wave_edit_drawing_area_button_release_select_buffer(GtkWidget *editor,
+							GtkWidget *toolbar,
 							AgsWaveEdit *wave_edit,
 							AgsMachine *machine,
 							GdkEventButton *event)
@@ -222,12 +298,26 @@ ags_wave_edit_drawing_area_button_release_select_buffer(AgsWaveEditor *wave_edit
 
   GtkAllocation allocation;
 
+  AgsApplicationContext *application_context;
+
+  gboolean use_composite_editor;
   gdouble c_range;
   guint g_range;
   double zoom_factor;
   guint x0, x1;
   gdouble y0, y1;
-    
+
+  application_context = ags_application_context_get_instance();
+
+  use_composite_editor = ags_ui_provider_use_composite_editor(AGS_UI_PROVIDER(application_context));
+  
+  /* zoom */
+  if(use_composite_editor){
+    zoom_factor = exp2(6.0 - (double) gtk_combo_box_get_active((GtkComboBox *) AGS_COMPOSITE_TOOLBAR(toolbar)->zoom));
+  }else{
+    zoom_factor = exp2(6.0 - (double) gtk_combo_box_get_active((GtkComboBox *) AGS_WAVE_TOOLBAR(toolbar)->zoom));
+  }
+  
   gtk_widget_get_allocation(GTK_WIDGET(wave_edit->drawing_area),
 			    &allocation);
 
@@ -237,9 +327,6 @@ ags_wave_edit_drawing_area_button_release_select_buffer(AgsWaveEditor *wave_edit
 
   g_range = gtk_adjustment_get_upper(vscrollbar_adjustment) + allocation.height;
 
-  /* zoom */
-  zoom_factor = exp2(6.0 - (double) gtk_combo_box_get_active((GtkComboBox *) wave_toolbar->zoom));
-
   /* region */
   x0 = (guint) wave_edit->selection_x0;
 
@@ -250,38 +337,86 @@ ags_wave_edit_drawing_area_button_release_select_buffer(AgsWaveEditor *wave_edit
   y1 = (((allocation.height - event->y) + gtk_range_get_value(GTK_RANGE(wave_edit->vscrollbar))) / g_range) * c_range;
     
   /* select region */
-  ags_wave_editor_select_region(wave_editor,
-				x0, y0,
-				x1, y1);
+  if(use_composite_editor){
+    ags_composite_editor_select_region(editor,
+				       x0, y0,
+				       x1, y1);
+  }else{
+    ags_wave_editor_select_region(editor,
+				  x0, y0,
+				  x1, y1);
+  }
 }
 
 gboolean
 ags_wave_edit_drawing_area_button_release_event(GtkWidget *widget, GdkEventButton *event, AgsWaveEdit *wave_edit)
 {
-  AgsWaveEditor *wave_editor;
-  AgsWaveToolbar *wave_toolbar;
+  GtkWidget *editor;
+  GtkWidget *toolbar;
   AgsMachine *machine;
 
-  wave_editor = (AgsWaveEditor *) gtk_widget_get_ancestor(GTK_WIDGET(wave_edit),
-							  AGS_TYPE_WAVE_EDITOR);
+  AgsApplicationContext *application_context;
   
-  wave_toolbar = wave_editor->wave_toolbar;
+  gboolean use_composite_editor;
+  gboolean selected_position_cursor, selected_select;
 
-  if((machine = wave_editor->selected_machine) != NULL &&
+  application_context = ags_application_context_get_instance();
+
+  use_composite_editor = ags_ui_provider_use_composite_editor(AGS_UI_PROVIDER(application_context));
+
+  editor = NULL;
+  toolbar = NULL;
+  
+  machine = NULL;
+  
+  selected_position_cursor = FALSE;
+  selected_select = FALSE;
+
+  if(use_composite_editor){
+    AgsCompositeToolbar *composite_toolbar;
+    
+    editor = gtk_widget_get_ancestor(GTK_WIDGET(wave_edit),
+				     AGS_TYPE_COMPOSITE_EDITOR);
+    
+    toolbar = AGS_COMPOSITE_EDITOR(editor)->toolbar;
+
+    machine = AGS_COMPOSITE_EDITOR(editor)->selected_machine;
+
+    composite_toolbar = (AgsCompositeToolbar *) toolbar;
+    
+    selected_position_cursor = (composite_toolbar->selected_tool == composite_toolbar->position) ? TRUE: FALSE;
+    selected_select = (composite_toolbar->selected_tool == composite_toolbar->select) ? TRUE: FALSE;
+  }else{
+    AgsWaveToolbar *wave_toolbar;
+    
+    editor = gtk_widget_get_ancestor(GTK_WIDGET(wave_edit),
+				     AGS_TYPE_WAVE_EDITOR);
+    
+    toolbar = AGS_WAVE_EDITOR(editor)->wave_toolbar;
+
+    machine = AGS_WAVE_EDITOR(editor)->selected_machine;
+
+    wave_toolbar = (AgsWaveToolbar *) toolbar;
+    
+    selected_position_cursor = (wave_toolbar->selected_edit_mode == wave_toolbar->position) ? TRUE: FALSE;
+    selected_select = (wave_toolbar->selected_edit_mode == wave_toolbar->select) ? TRUE: FALSE;
+  }
+  
+  if(machine != NULL &&
      event->button == 1){
     wave_edit->button_mask &= (~AGS_WAVE_EDIT_BUTTON_1);
     
-    if(wave_edit->mode == AGS_WAVE_EDIT_POSITION_CURSOR){
-      ags_wave_edit_drawing_area_button_release_position_cursor(wave_editor,
-								wave_toolbar,
+    if(selected_position_cursor){
+      ags_wave_edit_drawing_area_button_release_position_cursor(editor,
+								toolbar,
 								wave_edit,
 								machine,
 								event);
 
       wave_edit->mode = AGS_WAVE_EDIT_NO_EDIT_MODE;
-    }else if(wave_edit->mode == AGS_WAVE_EDIT_SELECT_BUFFER){
-      ags_wave_edit_drawing_area_button_release_select_buffer(wave_editor,
-							      wave_toolbar,
+    }else if(selected_select){
+      ags_wave_edit_drawing_area_button_release_select_buffer(editor,
+							      toolbar,
 							      wave_edit,
 							      machine,
 							      event);
@@ -294,8 +429,8 @@ ags_wave_edit_drawing_area_button_release_event(GtkWidget *widget, GdkEventButto
 }
 
 void
-ags_wave_edit_drawing_area_motion_notify_position_cursor(AgsWaveEditor *wave_editor,
-							 AgsWaveToolbar *wave_toolbar,
+ags_wave_edit_drawing_area_motion_notify_position_cursor(GtkWidget *editor,
+							 GtkWidget *toolbar,
 							 AgsWaveEdit *wave_edit,
 							 AgsMachine *machine,
 							 GdkEventMotion *event)
@@ -304,10 +439,24 @@ ags_wave_edit_drawing_area_motion_notify_position_cursor(AgsWaveEditor *wave_edi
     
   GtkAllocation allocation;
 
+  AgsApplicationContext *application_context;
+  
+  gboolean use_composite_editor;
   gdouble c_range;
   guint g_range;
   double zoom_factor;
 
+  application_context = ags_application_context_get_instance();
+
+  use_composite_editor = ags_ui_provider_use_composite_editor(AGS_UI_PROVIDER(application_context));
+
+  /* zoom */
+  if(use_composite_editor){
+    zoom_factor = exp2(6.0 - (double) gtk_combo_box_get_active((GtkComboBox *) AGS_COMPOSITE_TOOLBAR(toolbar)->zoom));
+  }else{
+    zoom_factor = exp2(6.0 - (double) gtk_combo_box_get_active((GtkComboBox *) AGS_WAVE_TOOLBAR(toolbar)->zoom));
+  }
+  
   gtk_widget_get_allocation(GTK_WIDGET(wave_edit->drawing_area),
 			    &allocation);
 
@@ -317,9 +466,6 @@ ags_wave_edit_drawing_area_motion_notify_position_cursor(AgsWaveEditor *wave_edi
 
   g_range = gtk_adjustment_get_upper(vscrollbar_adjustment) + allocation.height;
 
-  /* zoom */
-  zoom_factor = exp2(6.0 - (double) gtk_combo_box_get_active((GtkComboBox *) wave_toolbar->zoom));
-
   /* cursor position */
   wave_edit->cursor_position_x = ((zoom_factor * event->x + gtk_range_get_value(GTK_RANGE(wave_edit->hscrollbar))));
 
@@ -334,23 +480,34 @@ ags_wave_edit_drawing_area_motion_notify_position_cursor(AgsWaveEditor *wave_edi
 }
 
 void
-ags_wave_edit_drawing_area_motion_notify_select_buffer(AgsWaveEditor *wave_editor,
-						       AgsWaveToolbar *wave_toolbar,
+ags_wave_edit_drawing_area_motion_notify_select_buffer(GtkWidget *editor,
+						       GtkWidget *toolbar,
 						       AgsWaveEdit *wave_edit,
 						       AgsMachine *machine,
 						       GdkEventMotion *event)
 {
+  AgsApplicationContext *application_context;
+  
+  gboolean use_composite_editor;
   double zoom_factor;
 
-  /* zoom */
-  zoom_factor = exp2(6.0 - (double) gtk_combo_box_get_active((GtkComboBox *) wave_toolbar->zoom));
+  application_context = ags_application_context_get_instance();
 
+  use_composite_editor = ags_ui_provider_use_composite_editor(AGS_UI_PROVIDER(application_context));
+
+  /* zoom */
+  if(use_composite_editor){
+    zoom_factor = exp2(6.0 - (double) gtk_combo_box_get_active((GtkComboBox *) AGS_COMPOSITE_TOOLBAR(toolbar)->zoom));
+  }else{
+    zoom_factor = exp2(6.0 - (double) gtk_combo_box_get_active((GtkComboBox *) AGS_WAVE_TOOLBAR(toolbar)->zoom));
+  }
+  
   if(zoom_factor * event->x + gtk_range_get_value(GTK_RANGE(wave_edit->hscrollbar)) >= 0.0){
     wave_edit->selection_x1 = (guint) zoom_factor * event->x + gtk_range_get_value(GTK_RANGE(wave_edit->hscrollbar));
   }else{
     wave_edit->selection_x1 = 0.0;
   }
-    
+  
   if(event->y + gtk_range_get_value(GTK_RANGE(wave_edit->vscrollbar)) >= 0.0){
     wave_edit->selection_y1 = (guint) event->y + gtk_range_get_value(GTK_RANGE(wave_edit->vscrollbar));
   }else{
@@ -363,28 +520,65 @@ ags_wave_edit_drawing_area_motion_notify_select_buffer(AgsWaveEditor *wave_edito
 gboolean
 ags_wave_edit_drawing_area_motion_notify_event(GtkWidget *widget, GdkEventMotion *event, AgsWaveEdit *wave_edit)
 {
-  AgsWaveEditor *wave_editor;
-  AgsWaveToolbar *wave_toolbar;
+  GtkWidget *editor;
+  GtkWidget *toolbar;
   AgsMachine *machine;
 
-  wave_editor = (AgsWaveEditor *) gtk_widget_get_ancestor(GTK_WIDGET(wave_edit),
-							  AGS_TYPE_WAVE_EDITOR);
+  AgsApplicationContext *application_context;
+  
+  gboolean use_composite_editor;
+  gboolean selected_position_cursor, selected_edit, selected_clear, selected_select;
 
-  wave_toolbar = wave_editor->wave_toolbar;
+  application_context = ags_application_context_get_instance();
 
+  use_composite_editor = ags_ui_provider_use_composite_editor(AGS_UI_PROVIDER(application_context));
+
+  selected_position_cursor = FALSE;
+  selected_select = FALSE;
+
+  if(use_composite_editor){
+    AgsCompositeToolbar *composite_toolbar;
+    
+    editor = gtk_widget_get_ancestor(GTK_WIDGET(wave_edit),
+				     AGS_TYPE_COMPOSITE_EDITOR);
+    
+    toolbar = AGS_COMPOSITE_EDITOR(editor)->toolbar;
+
+    machine = AGS_COMPOSITE_EDITOR(editor)->selected_machine;
+
+    composite_toolbar = (AgsCompositeToolbar *) toolbar;
+
+    selected_position_cursor = (composite_toolbar->selected_tool == composite_toolbar->position) ? TRUE: FALSE;
+    selected_select = (composite_toolbar->selected_tool == composite_toolbar->select) ? TRUE: FALSE;
+  }else{
+    AgsWaveToolbar *wave_toolbar;
+    
+    editor = gtk_widget_get_ancestor(GTK_WIDGET(wave_edit),
+				     AGS_TYPE_WAVE_EDITOR);
+    
+    toolbar = AGS_WAVE_EDITOR(editor)->wave_toolbar;
+
+    machine = AGS_WAVE_EDITOR(editor)->selected_machine;
+
+    wave_toolbar = (AgsWaveToolbar *) toolbar;
+    
+    selected_position_cursor = (wave_toolbar->selected_edit_mode == wave_toolbar->position) ? TRUE: FALSE;
+    selected_select = (wave_toolbar->selected_edit_mode == wave_toolbar->select) ? TRUE: FALSE;
+  }
+  
   gtk_widget_grab_focus((GtkWidget *) wave_edit->drawing_area);
 
-  if((machine = wave_editor->selected_machine) != NULL &&
+  if(machine != NULL &&
      (AGS_WAVE_EDIT_BUTTON_1 & (wave_edit->button_mask)) != 0){
     if(wave_edit->mode == AGS_WAVE_EDIT_POSITION_CURSOR){
-      ags_wave_edit_drawing_area_motion_notify_position_cursor(wave_editor,
-							       wave_toolbar,
+      ags_wave_edit_drawing_area_motion_notify_position_cursor(editor,
+							       toolbar,
 							       wave_edit,
 							       machine,
 							       event);
     }else if(wave_edit->mode == AGS_WAVE_EDIT_SELECT_BUFFER){
-      ags_wave_edit_drawing_area_motion_notify_select_buffer(wave_editor,
-							     wave_toolbar,
+      ags_wave_edit_drawing_area_motion_notify_select_buffer(editor,
+							     toolbar,
 							     wave_edit,
 							     machine,
 							     event);
@@ -397,9 +591,12 @@ ags_wave_edit_drawing_area_motion_notify_event(GtkWidget *widget, GdkEventMotion
 gboolean
 ags_wave_edit_drawing_area_key_press_event(GtkWidget *widget, GdkEventKey *event, AgsWaveEdit *wave_edit)
 {
-  AgsWaveEditor *wave_editor;
+  GtkWidget *editor;
   AgsMachine *machine;
 
+  AgsApplicationContext *application_context;
+  
+  gboolean use_composite_editor;
   gboolean retval;
   
   if(event->keyval == GDK_KEY_Tab ||
@@ -415,10 +612,23 @@ ags_wave_edit_drawing_area_key_press_event(GtkWidget *widget, GdkEventKey *event
     retval = TRUE;
   }
 
-  wave_editor = (AgsWaveEditor *) gtk_widget_get_ancestor(GTK_WIDGET(wave_edit),
-							  AGS_TYPE_WAVE_EDITOR);
+  application_context = ags_application_context_get_instance();
 
-  machine = wave_editor->selected_machine;
+  use_composite_editor = ags_ui_provider_use_composite_editor(AGS_UI_PROVIDER(application_context));
+
+  machine = NULL;
+  
+  if(use_composite_editor){
+    editor = gtk_widget_get_ancestor(GTK_WIDGET(wave_edit),
+				     AGS_TYPE_COMPOSITE_EDITOR);
+    
+    machine = AGS_COMPOSITE_EDITOR(editor)->selected_machine;
+  }else{
+    editor = gtk_widget_get_ancestor(GTK_WIDGET(wave_edit),
+				     AGS_TYPE_WAVE_EDITOR);
+    
+    machine = AGS_WAVE_EDITOR(editor)->selected_machine;
+  }
   
   if(machine != NULL){
     switch(event->keyval){
@@ -446,7 +656,11 @@ ags_wave_edit_drawing_area_key_press_event(GtkWidget *widget, GdkEventKey *event
       {
 	/* select all accelerations */
 	if((AGS_WAVE_EDIT_KEY_L_CONTROL & (wave_edit->key_mask)) != 0 || (AGS_WAVE_EDIT_KEY_R_CONTROL & (wave_edit->key_mask)) != 0){
-	  ags_wave_editor_select_all(wave_editor);
+	  if(use_composite_editor){
+	    ags_composite_editor_select_all(editor);
+	  }else{
+	    ags_wave_editor_select_all(editor);
+	  }
 	}
       }
       break;
@@ -454,7 +668,11 @@ ags_wave_edit_drawing_area_key_press_event(GtkWidget *widget, GdkEventKey *event
       {
 	/* copy accelerations */
 	if((AGS_WAVE_EDIT_KEY_L_CONTROL & (wave_edit->key_mask)) != 0 || (AGS_WAVE_EDIT_KEY_R_CONTROL & (wave_edit->key_mask)) != 0){
-	  ags_wave_editor_copy(wave_editor);
+	  if(use_composite_editor){
+	    ags_composite_editor_copy(editor);
+	  }else{
+	    ags_wave_editor_copy(editor);
+	  }
 	}
       }
       break;
@@ -462,7 +680,11 @@ ags_wave_edit_drawing_area_key_press_event(GtkWidget *widget, GdkEventKey *event
       {
 	/* paste accelerations */
 	if((AGS_WAVE_EDIT_KEY_L_CONTROL & (wave_edit->key_mask)) != 0 || (AGS_WAVE_EDIT_KEY_R_CONTROL & (wave_edit->key_mask)) != 0){
-	  ags_wave_editor_paste(wave_editor);
+	  if(use_composite_editor){
+	    ags_composite_editor_paste(editor);
+	  }else{
+	    ags_wave_editor_paste(editor);
+	  }
 	}
       }
       break;
@@ -470,7 +692,11 @@ ags_wave_edit_drawing_area_key_press_event(GtkWidget *widget, GdkEventKey *event
       {
 	/* cut accelerations */
 	if((AGS_WAVE_EDIT_KEY_L_CONTROL & (wave_edit->key_mask)) != 0 || (AGS_WAVE_EDIT_KEY_R_CONTROL & (wave_edit->key_mask)) != 0){
-	  ags_wave_editor_cut(wave_editor);
+	  if(use_composite_editor){
+	    ags_composite_editor_cut(editor);
+	  }else{
+	    ags_wave_editor_cut(editor);
+	  }
 	}
       }
       break;
@@ -478,16 +704,26 @@ ags_wave_edit_drawing_area_key_press_event(GtkWidget *widget, GdkEventKey *event
       {
 	/* meta */
 	if((AGS_WAVE_EDIT_KEY_L_CONTROL & (wave_edit->key_mask)) != 0 || (AGS_WAVE_EDIT_KEY_R_CONTROL & (wave_edit->key_mask)) != 0){
-	  if((AGS_WAVE_META_ENABLED & (wave_editor->wave_meta->flags)) != 0){
-	    wave_editor->wave_meta->flags &= (~AGS_WAVE_META_ENABLED);
+	  AgsWaveMeta *wave_meta;
 
-	    gtk_widget_hide(wave_editor->wave_meta);
+	  wave_meta = NULL;
+	  
+	  if(use_composite_editor){
+	    wave_meta = AGS_COMPOSITE_EDITOR(editor)->wave_edit->edit_meta;
 	  }else{
-	    wave_editor->wave_meta->flags |= AGS_WAVE_META_ENABLED;
+	    wave_meta = AGS_WAVE_EDITOR(editor)->wave_meta;
+	  }
+	  
+	  if((AGS_WAVE_META_ENABLED & (wave_meta->flags)) != 0){
+	    wave_meta->flags &= (~AGS_WAVE_META_ENABLED);
 
-	    gtk_widget_show_all(wave_editor->wave_meta);
+	    gtk_widget_hide(wave_meta);
+	  }else{
+	    wave_meta->flags |= AGS_WAVE_META_ENABLED;
 
-	    ags_wave_meta_refresh(wave_editor->wave_meta);
+	    gtk_widget_show_all(wave_meta);
+
+	    ags_wave_meta_refresh(wave_meta);
 	  }
 	}
       }
@@ -501,25 +737,41 @@ ags_wave_edit_drawing_area_key_press_event(GtkWidget *widget, GdkEventKey *event
 gboolean
 ags_wave_edit_drawing_area_key_release_event(GtkWidget *widget, GdkEventKey *event, AgsWaveEdit *wave_edit)
 {
-  AgsWaveEditor *wave_editor;
-  AgsWaveToolbar *wave_toolbar;
+  GtkWidget *editor;
+  GtkWidget *toolbar;
   AgsMachine *machine;
-
+  AgsNotebook *channel_selector;
+  
+  AgsApplicationContext *application_context;
+  
   GtkAllocation allocation;
 
+  gboolean use_composite_editor;
   double zoom_factor;
   gint i;
   gboolean retval;
 
-  wave_editor = (AgsWaveEditor *) gtk_widget_get_ancestor(GTK_WIDGET(wave_edit),
-							  AGS_TYPE_WAVE_EDITOR);
+  application_context = ags_application_context_get_instance();
 
-  wave_toolbar = wave_editor->wave_toolbar;
+  use_composite_editor = ags_ui_provider_use_composite_editor(AGS_UI_PROVIDER(application_context));
 
-  machine = wave_editor->selected_machine;
+  machine = NULL;
+  
+  if(use_composite_editor){
+    editor = gtk_widget_get_ancestor(GTK_WIDGET(wave_edit),
+				     AGS_TYPE_COMPOSITE_EDITOR);
 
-  gtk_widget_get_allocation(GTK_WIDGET(wave_edit->drawing_area),
-			    &allocation);
+    channel_selector = AGS_COMPOSITE_EDITOR(editor)->wave_edit->channel_selector;
+    
+    machine = AGS_COMPOSITE_EDITOR(editor)->selected_machine;  
+  }else{
+    editor = gtk_widget_get_ancestor(GTK_WIDGET(wave_edit),
+				     AGS_TYPE_WAVE_EDITOR);
+    
+    channel_selector = AGS_WAVE_EDITOR(editor)->notebook;
+
+    machine = AGS_WAVE_EDITOR(editor)->selected_machine;
+  }
   
   if(event->keyval == GDK_KEY_Tab ||
      event->keyval == GDK_KEY_ISO_Left_Tab ||
@@ -534,10 +786,16 @@ ags_wave_edit_drawing_area_key_release_event(GtkWidget *widget, GdkEventKey *eve
     retval = TRUE;
   }
 
-  if(machine != NULL){    
-    /* zoom */
-    zoom_factor = exp2(6.0 - (double) gtk_combo_box_get_active((GtkComboBox *) wave_toolbar->zoom));
+  if(use_composite_editor){
+    zoom_factor = exp2(6.0 - (double) gtk_combo_box_get_active((GtkComboBox *) AGS_COMPOSITE_TOOLBAR(AGS_COMPOSITE_EDITOR(editor)->toolbar)->zoom));
+  }else{
+    zoom_factor = exp2(6.0 - (double) gtk_combo_box_get_active((GtkComboBox *) AGS_WAVE_TOOLBAR(AGS_WAVE_EDITOR(editor)->wave_toolbar)->zoom));
+  }
 
+  gtk_widget_get_allocation(GTK_WIDGET(wave_edit->drawing_area),
+			    &allocation);
+
+  if(machine != NULL){    
     /* check key value */
     switch(event->keyval){
     case GDK_KEY_Control_L:
@@ -618,13 +876,18 @@ ags_wave_edit_vscrollbar_value_changed(GtkRange *range, AgsWaveEdit *wave_edit)
 void
 ags_wave_edit_hscrollbar_value_changed(GtkRange *range, AgsWaveEdit *wave_edit)
 {
+  GtkWidget *editor;
+  
   AgsApplicationContext *application_context;
   
+  gboolean use_composite_editor;
   gdouble gui_scale_factor;
   gdouble value;
 
   application_context = ags_application_context_get_instance();
 
+  use_composite_editor = ags_ui_provider_use_composite_editor(AGS_UI_PROVIDER(application_context));
+
   /* scale factor */
   gui_scale_factor = ags_ui_provider_get_gui_scale_factor(AGS_UI_PROVIDER(application_context));
 
@@ -632,6 +895,15 @@ ags_wave_edit_hscrollbar_value_changed(GtkRange *range, AgsWaveEdit *wave_edit)
   gtk_adjustment_set_value(wave_edit->ruler->adjustment,
 			   gui_scale_factor * value);
   gtk_widget_queue_draw((GtkWidget *) wave_edit->ruler);
+
+  if(use_composite_editor){
+    editor = gtk_widget_get_ancestor(GTK_WIDGET(wave_edit),
+				     AGS_TYPE_COMPOSITE_EDITOR);
+    
+    gtk_adjustment_set_value(AGS_COMPOSITE_EDITOR(editor)->wave_edit->ruler->adjustment,
+			     gui_scale_factor * value);
+    gtk_widget_queue_draw((GtkWidget *) AGS_COMPOSITE_EDITOR(editor)->wave_edit->ruler);
+  }
   
   /* queue draw */
   gtk_widget_queue_draw((GtkWidget *) wave_edit->drawing_area);
